#include "inkey.ch"

#define K_VIRTUAL_COL     1                      && the number of the "virtual" column

#define K_CARGO_ELEMENTS  1

STATIC oBr

FUNCTION main()
  LOCAL nRow    := ROW()
  LOCAL nCol    := COL()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )

  SETCOLOR("+BG/B")
  CLEAR
  USE Customer INDEX Customer NEW SHARED
  LoadBrowse()

  SET CURSOR OFF
  KeyProcess()
  SET CURSOR ON

  CLOSE DATABASES
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)
  SET COLOR TO
  DEVPOS(nRow, nCol)
RETURN(NIL)

***********
* Function: LoadBrowse()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Create & loads a TBrowse object
* Returns.: NIL
***********

FUNCTION LoadBrowse()

  SETCOLOR("+BG/N")
  oBr := TBrowseDB(3, 23, 21, 65 )              && Generic DBF skipper included already

  oBr:Cargo                 := ARRAY(K_CARGO_ELEMENTS)
  oBr:Cargo[K_VIRTUAL_COL]  := 1

  oBr:AddColumn(TBColumnNew("Column  1", {|| CustomerData()} ))  &&   assign it a single column
  @ 1, 0 SAY PADC('BROWSE OF MANY "VIRTUAL" COLUMNS', 80 ) COLOR "+GR/B"

RETURN(NIL)


***********
* Function: KeyProcess
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Processes keystrokes for a TBrowse
*
* Params..: none
*
* Returns.: NIL
***********

FUNCTION KeyProcess()
  LOCAL nKey                                     && Key that the user presses

  DO WHILE .T.
    nKey := 0                                    && Reset value of nKey

    *** stabilize the display (standard TBrowse stuff) ***
    DO WHILE .NOT. oBr:stabilize()               && DO while the display is not stabilized, incrementally stabilize it.
      nKey := INKEY()                            &&   check to see if the user has pressed a key
      IF nKey <> 0                               &&   IF a key was pressed
        EXIT                                     &&     EXIT out of this loop and process the user's request
      ENDIF                                      &&   ENDIF
    ENDDO                                        && ENDDO - go back and stabilize it a little more...

    *** This part is needed for multiuser applications to ensure that we are displaying the most current version of the data ***
    IF nKey == 0                                 && IF we disable caching AND the browse was completely re-drawn without a user request
      oBr:RefreshCurrent()                       &&   mark that the "Current" line needs to be refreshed
      DO WHILE .NOT. oBr:stabilize()             &&   re-draw (hopefully just 1 line)
      ENDDO                                      &&   no need to check keyboard etc.
    ENDIF                                        && ENDIF

    IF oBr:stable .AND. nKey==0                  && IF the display is stable
      nKey := INKEY(0)                           &&   Wait forever for the user to hit a key
    ENDIF                                        && ENDIF

    *** Finally, we get to process the key that was pressed ***
    DO CASE
      CASE nKey == K_DOWN
        oBr:down()

      CASE nKey == K_UP
        oBr:up()

      CASE nKey == K_PGDN
        oBr:pageDown()

      CASE nKey == K_PGUP
        oBr:pageUp()

      CASE nKey == K_ESC
        EXIT

      CASE nKey == K_ENTER .AND. oBr:Stable
        oBr:DeHiLite()
        EditCustomer()
        oBr:RefreshAll()                         && re-draw entire TBrowse after editing the record

      CASE nKey == K_RIGHT .AND. oBr:Stable
        oBr:Cargo[K_VIRTUAL_COL] := MIN(99, oBr:Cargo[K_VIRTUAL_COL]+1)
        oBr:GetColumn(1):Heading := "Column "+STR(oBr:Cargo[K_VIRTUAL_COL], 2)
        oBr:Configure()

      CASE nKey == K_LEFT .AND. oBr:Stable
        oBr:Cargo[K_VIRTUAL_COL] := MAX(1, oBr:Cargo[K_VIRTUAL_COL]-1)
        oBr:GetColumn(1):Heading := "Column "+STR(oBr:Cargo[K_VIRTUAL_COL], 2)
        oBr:Configure()

    ENDCASE
  ENDDO
RETURN(NIL)

***********
* Function: EditCustomer()
* Author..: Troy Henkoff
* Date....: 10/17/92
* Notes...: edits on customer record
* Returns.: NIL
***********

FUNCTION EditCustomer()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )
  LOCAL cColor  := SETCOLOR( "+W/N" )
  LOCAL getlist := {}

  SET CURSOR ON
  @ 17, 1, 23, 49 BOX "Ŀ "          COLOR " BG/N"  && Draw pretty box for user
  @ 17, 4 SAY " Editing Customer Record "  COLOR "+GR/N"  && put a title on it

  IF rlock()                                     && IF we can lock the record
    @ 18,  2 SAY "Customer......:" GET Customer PICTURE "999999"  &&   Let user edit data...
    @ 19,  2 SAY "Name..........:" GET Name     PICTURE "@!"
    @ 20,  2 SAY "Address.......:" GET Addr_1
    @ 21,  2 SAY "              :" GET Addr_2
    @ 22,  2 SAY "City, St, Zip.:" GET City
    @ 22, 34                       GET State    PICTURE "!!"
    @ 22, 38                       GET Zip      PICTURE "99999-9999"
    READ
    COMMIT                                       &&   NOTE: Flush Clipper's buffers BEFORE UNLOCK
    UNLOCK                                       &&   UNLOCK the record
  ELSE                                           && ELSE
    @20,10 SAY "Record not available..." COLOR "+R/N"  &&   Someone else has the record...
    ?? CHR(7)
    INKEY(3)
  ENDIF                                          && ENDIF
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)  && restore the screen, etc...
  SETCOLOR(cColor)
  SET CURSOR OFF

RETURN(NIL)


***********
* Function: CustomerData()
* Author..: Troy Henikoff
* Date....: 10/24/92    
* Returns.: STRING of the virtual column for a customer
***********

FUNCTION CustomerData()
  LOCAL cString

  IF oBr:Cargo[K_VIRTUAL_COL] == 1
    cString := Customer->Customer+" : "+Customer->Name 
  ELSE
    cString := Customer->Addr_1+"Data"+SlowFunc(oBr:Cargo[K_VIRTUAL_COL]) 
  ENDIF
RETURN(cString)

***********
* Function: SlowFunc()
* Author..: Troy Henikoff
* Date....: 10/24/92    
* Notes...: Converts a number to a character string, slowly...
* 
* Params..: nParam - NUMBER to be converted
* 
* Returns.: STRING of the number
***********

FUNCTION SlowFunc
  PARAMETERS nParam                              && "Private" parameter, it's slower...
  PRIVATE nTemp, cString                         && "Private" vars, to slow us down..
  PRIVATE nCounter

  FOR nCounter := 1 TO 10
    nTemp := nParam * 1000
    nTemp := nTemp / 10
    nTemp := nTemp / 10
    nTemp := nTemp / 10
  NEXT

  cString := RIGHT( STR(nTemp,10), 3) + "Some more string manipulations..."
  cString := LEFT( cString, 3)
RETURN(cString)
